/*=========================================================================

  Program:   UsimagToolBase
  Language:  C++
  Date:      5-07-2007
  Version:   1.0

  Copyright (c) 2007 Laboratoy of Image Processing, UVA. All rights reserved.
  See http://www.lpi.tel.uva.es/UsimagTool for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE. 

=========================================================================*/

#include <fstream>
#include "UsimagToolBase.h"
#include "FL/fl_ask.H"

UsimagToolBase
::UsimagToolBase()
{

  //Readers 
  m_RawImageReader  =  FileReaderType::New();
  m_MetaImageReader =  FileReaderType::New();
  m_MetaVectorImageReader =  FileVectorReaderType::New();
  m_JPEGImageReader =  FileReaderType::New();
  m_VTKImageReader =  FileReaderType::New();
  m_TIFFImageReader =  FileReaderType::New();
  m_PNGImageReader  =  FileReaderType::New();
  m_DicomImageReader =  FileFloatReaderType::New();
  m_DicomSeriesImageReader =  SeriesReaderType::New();
  m_GenericReader   =  FileReaderType::New();
  m_JPEGFloatImageReader = FileFloat2DReaderType::New();

  m_RawReader       =  RawReaderType::New(); 
  m_MetaReader      =  MetaImageIOType::New();   
  m_JPEGReader      =  JPEGImageIOType::New();
  m_TIFFReader      =  TIFFImageIOType::New(); 
  m_PNGReader       =  PNGImageIOType::New(); 
  m_DicomReader     =  DicomImageIOType::New(); 
  m_DicomSeriesReader = DicomSeriesImageIOType::New();
  m_nameGenerator = itk::DICOMSeriesFileNames::New();

  m_RawImageReader->SetImageIO( m_RawReader );
  m_MetaImageReader->SetImageIO( m_MetaReader );
  m_JPEGImageReader->SetImageIO( m_JPEGReader );
  m_TIFFImageReader->SetImageIO( m_TIFFReader );
  m_PNGImageReader->SetImageIO( m_PNGReader );
  m_DicomImageReader->SetImageIO( m_DicomReader );
  m_DicomSeriesImageReader->SetImageIO( m_DicomSeriesReader );

  //Writers 
  m_ImageWriter       =  FloatWriterType::New();  
  m_MetaImageWriter   =  FloatWriterType::New(); 
  m_FloatImageWriter  =  FloatWriterType::New(); 
  m_RawWriter         =  RawReaderType::New(); 
  m_MetaWriter        =  MetaImageIOType::New(); 
  
  m_MetaImageWriter->SetImageIO( m_MetaWriter );
  
  /// PARA VTK
  m_VTKviewer         = VTKImageViewerType::New();
  m_connector         = ConnectorType::New();
  //connector->GetInput(VTKreader->GetOutput);

  m_VTKexporter = VTKexportType::New(); 

  m_vtkMarcacionElipsoide = vtkMarcacionElipsoide::New();
  m_vtkPlantillaAjustada = vtkPlantillaAjustada::New(); 
  m_vtkFuncionVerosimilitud = vtkFuncionVerosimilitud::New();
  m_vtkOptimizaContorno = vtkOptimizaContorno::New();
  m_vtkValidacion = vtkValidacion::New();

  // Filtros:
  m_MeanFilter = MeanFilterType::New();
  m_BilateralFilter = BilateralFilterType::New();
  m_GradientDiffusionFilter =  GradientDiffusionType::New();
  m_GradientFilter =  GradientFilterType::New();
  m_LevelSetSegmentation = SegmentationFilterType::New();
  //m_FuzzyConnectSeg = FuzzyConnectFilterType::New();
  m_char_to_real = CastCharToRealFilterType::New();
  m_real_to_char = CastRealToCharFilterType::New();
  m_2Dto3DCaster = Cast2Dto3DFilterType::New();
  m_3Dto2DCaster = Cast3Dto2DFilterType::New();
  m_SegmentedCaster = Cast2Dto3DFilterType::New();
  m_InputCaster = Cast3Dto2DFilterType::New();
  m_OutputCaster = Cast2Dto3DFilterType::New();

  m_fastMarching = FastMarchingFilterType::New();
  m_Curvature  = CurvatureDiffusionType::New();
  m_Curvature2D  = Curvature2DDiffusionType::New();
  m_thresholder  = BinaryThresholdType::New();
  m_GaussianFilter = GaussianFilterType::New(); 
  m_MedianFilter = MedianFilterType::New(); 
  //m_WatershedFilter = WatershedFilterType::New();
  m_ThresholdFilter = BinaryThresholdFilterType::New();
  m_AddFilter = AddFilterType::New();
  m_MultiplyFilter = MultiplyFilterType::New();
  m_RelabelFilter = RelabelFilterType::New();
  m_RescaleFilter = RescaleFilterType::New();
  m_RelabelCompFilter = RelabelCompFilterType::New();  
  m_WienerFilter = WienerFilterType::New();
  m_ASRFilter = ASRFilterType::New();
  m_ASRFilter2D = ASRFilterType2D::New();
  m_SRADFilter = SRADFilterType::New();
  m_SRADdiffusion = SRADDiffusionType::New();        
  m_DPADFilter = DPADFilterType::New();
  //m_DPADdiffusion = DPADDiffusionType::New();        
  m_regMLD = RegisterMLDType::New(); 
  m_ABSFilter = ABSFilterType::New();
  m_ABSValDifFilter = ABSValDifFilterType::New();
  m_InvertFilter = InvertFilterType::New();
  m_ZeroEdgeFilter = ZeroEdgeFilterType::New();

  m_structuringElement.SetRadius( 1 );  // 3x3 structuring element
  m_structuringElement.CreateStructuringElement();
  m_ErodeFilter = ErodeFilterType::New();
  m_DilateFilter = DilateFilterType::New();
  m_KnnFilter = classifyKNNCoreType::New(); 

  m_CannyFilter = CannyFilterType::New();

  //Parametros generales
  m_NImagesCargadas = 0;
  m_activeinput = 0;

  m_NumberOfPixelsInX = 256;
  m_NumberOfPixelsInY = 256;
  m_NumberOfPixelsInZ = 1;

  m_SpacingX = 1;
  m_SpacingY = 1;
  m_SpacingZ = 1;

  //Parametros especificos
  m_NIterations = 10;
  m_TimeStep = 1;
  m_Conductance = 0.1;
  // Segmentacion 
  m_threshold = 100;


  // Imagenes
  m_InputImage[0] = InputImageType::New();
  m_InputImage[1] = InputImageType::New();
  m_InputImage[2] = InputImageType::New();
  m_InputImage[3] = InputImageType::New();
  m_FloatInputImage = FloatImageType2D::New();
  m_VectorImage = VectorImageType::New();

  m_Size[0] = 256; m_Size[1] = 256; m_Size[2] = 1; 
  m_RegionInit.SetSize(m_Size);
  for (int i=0; i < 4; i++ ) {
    m_InputImage[i]->SetRegions(m_RegionInit);
    m_InputImage[i]->Allocate();
    m_InputImage[i]->FillBuffer(0);
  }

  m_OutputImage = InputImageType::New();
  m_FloatImage  = FloatImageType::New();
  m_FloatImage2  = FloatImageType::New();

  //// Para reservar memoria para una imagen ////
  //ImageType::SizeType imSize = {{256, 256, 10}};
  //ImageType::Pointer im = ImageType::New();
  //im->SetRegions(imSize);
  //im->Allocate();

}

UsimagToolBase::~UsimagToolBase()
{
}

void UsimagToolBase::SetDimensionX( unsigned int numberOfPixels )
{
  m_NumberOfPixelsInX = numberOfPixels;
}


void UsimagToolBase::SetDimensionY( unsigned int numberOfPixels )
{
  m_NumberOfPixelsInY = numberOfPixels;
}


void UsimagToolBase::SetDimensionZ( unsigned int numberOfPixels )
{
  m_NumberOfPixelsInZ = numberOfPixels;
}

void UsimagToolBase::SetSpacingX( double value )
{
  m_SpacingX = value;
}

void UsimagToolBase::SetSpacingY( double value )
{
  m_SpacingY = value;
}

void UsimagToolBase::SetSpacingZ( double value )
{
  m_SpacingZ = value;
}

void UsimagToolBase::SetNIterations( unsigned int iterations )
{
  m_NIterations = iterations;
}

void UsimagToolBase::SetConductance( double conductance )
{
  m_Conductance = conductance;
}

void UsimagToolBase::SetTimeStep( MyTimeStepType timestep )
{
  m_TimeStep = timestep;
}

//void UsimagToolBase::SetThresholdFuzzy( double value) {
//  m_threshold = value;
//}

void UsimagToolBase::SetInMean( double value) {
  m_inmean = value;
}

void UsimagToolBase::SetInVar( double value) {
  m_invar = value;
}

void UsimagToolBase::SetInDifMean( double value) {
  m_indifmean = value;
}

void UsimagToolBase::SetInDifVar( double value) {
  m_indifvar = value;
}

void UsimagToolBase::SetInWeight( double value) {
  m_inweight = value;
}

void UsimagToolBase::ShowImageOut(void)
{
 
  std::cout << "m_OutputImage size: " << m_OutputImage->GetLargestPossibleRegion().GetSize() << std::endl;
  m_ViewerOut.SetLabel("Resultado");
  m_ViewerOut.SetImage( m_OutputImage );
  m_ViewerOut.Show();
}

void UsimagToolBase::ShowVTK(void)
{
  m_VTKviewer->SetImage(m_InputImage[0]);
  m_VTKviewer->Show();
}

void UsimagToolBase::Save( void )
{
  
  m_filename = fl_file_chooser("Image Filename","*","");
 
  if( !m_filename )
    {
    return;
    }

  this->Save( m_filename );
}

void UsimagToolBase::Save( const char * filename )
{

  m_ImageWriter->SetFileName( filename );
  m_ImageWriter->SetInput( m_InputImage[m_activeinput]);

  // Attempt to Write
  m_ImageWriter->Write();
}

void UsimagToolBase::SaveFloat( void )
{
  
  m_filename = fl_file_chooser("Image Filename","*","");
 
  if( !m_filename )
    {
    return;
    }

  this->SaveFloat( m_filename );
}

void UsimagToolBase::SaveFloat( const char * filename )
{

  m_FloatImageWriter->SetFileName( filename );
  m_FloatImageWriter->SetInput( m_DicomSeriesImageReader->GetOutput());

  // Attempt to Write
  try {
    m_FloatImageWriter->Write();
  } catch (itk::ExceptionObject &ex) {
      std::cout << ex << std::endl;
  }
  
}



