/*=========================================================================

  Program:   UsimagToolBase
  Language:  C++
  Date:      5-07-2007
  Version:   1.0

  Copyright (c) 2007 Laboratoy of Image Processing, UVA. All rights reserved.
  See http://www.lpi.tel.uva.es/UsimagTool for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE. 

=========================================================================*/

#include <itkImage.h>
#include <itkRawImageIO.h>
#include <itkMetaImageIO.h>
#include <itkJPEGImageIO.h>
#include <itkTIFFImageIO.h>
#include <itkPNGImageIO.h>
#include <itkDicomImageIO.h>
#include <itkImageSeriesReader.h>
#include <itkDICOMImageIO2.h>
#include <itkDICOMSeriesFileNames.h>
#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>
#include <itkCastImageFilter.h>
//#include <itkAnisotropicDiffusionImageFilter.h>
#include <itkBilateralImageFilter.h>
#include <itkGradientMagnitudeImageFilter.h>
#include <itkBinaryThresholdImageFilter.h>
#include <itkThresholdSegmentationLevelSetImageFilter.h>
#include <itkFastMarchingImageFilter.h>
//#include <itkSimpleFuzzyConnectednessScalarImageFilter.h>
#include <itkCurvatureAnisotropicDiffusionImageFilter.h>
#include <itkGradientAnisotropicDiffusionImageFilter.h>
#include <itkBinaryThresholdImageFilter.h>
#include <itkRecursiveGaussianImageFilter.h>
#include <itkMedianImageFilter.h>
#include <itkMeanImageFilter.h>
#include <itkWatershedImageFilter.h>
#include <itkChangeLabelImageFilter.h>
#include <itkRescaleIntensityImageFilter.h>
#include <itkRelabelComponentImageFilter.h>
#include <itkErodeObjectMorphologyImageFilter.h>
#include <itkDilateObjectMorphologyImageFilter.h>
#include <itkBinaryBallStructuringElement.h>
#include <itkMultiplyImageFilter.h>
#include <itkAddImageFilter.h>
#include <itkImageRegionConstIterator.h>
#include <itkImageRegionIterator.h>
#include <itkVTKImageExport.h>
#include <itkAbsImageFilter.h>
#include <itkVector.h>
#include <itkAbsoluteValueDifferenceImageFilter.h>
#include <itkInvertIntensityImageFilter.h>
#include <itkZeroCrossingBasedEdgeDetectionImageFilter.h>
#include <itkCannyEdgeDetectionImageFilter.h>



#include "knn-1canal/classifyKNNCore.h"
#include "knn-1canal/voronoiFilter.h"
#include "Kretz/itkImageIOFactory.h"
#include "VtkFltk/fltkVTKImageViewer.h"
#include "wiener/itkWienerFilter.h"
#include "ASR/itkASRFilter.h"
#include "SRAD/itkSRADFilter.h"
#include "DPAD/itkDPADFilter.h"
#include "Demons3D/itkMLD.h"

#include "MyfltkImageViewer/MyfltkImageViewer.h"
#include "MyfltkImageViewer/GLSliceView.h"
#include "MyfltkImageViewer/ImageViewer.h"
#include <FL/Fl_File_Chooser.H>

#include "vtkMarcacionElipse/vtkMarcacionElipsoide.h"
#include "vtkMarcacionElipse/vtkPlantillaAjustada.h"
#include "vtkMarcacionElipse/vtkFuncionVerosimilitud.h"
#include "vtkMarcacionElipse/vtkOptimizaContorno.h"
#include "vtkMarcacionElipse/vtkValidacion.h"

class UsimagToolBase
{
public:

  enum { Dimension =  3 };

  //typedef unsigned char                              InputPixelType;
  typedef float                                      InputPixelType;
  typedef unsigned char                              UCharPixelType;
  typedef float                                      FloatPixelType;
  typedef itk::Image< InputPixelType, Dimension >    InputImageType;
  typedef itk::Image< InputPixelType, 2 >            InputImageType2D;
  typedef itk::Image< UCharPixelType, Dimension >    UCharImageType;
  typedef itk::Image< FloatPixelType, Dimension >    FloatImageType;
  typedef itk::Image< FloatPixelType, 2 >            FloatImageType2D;
  // Soporte imagenes vectoriales 
  typedef itk::Vector<float, 2> VectorPixelType;
  typedef itk::Image< VectorPixelType, 2 >    VectorImageType;

  typedef InputImageType::RegionType                 InputRegionType;
  typedef InputImageType::RegionType::SizeType       SizeType;
  typedef InputImageType::IndexType                  InputIndexType;
  // Readers: 
  typedef itk::ImageFileReader< InputImageType >     FileReaderType;
  typedef itk::ImageFileReader< FloatImageType >     FileFloatReaderType;
  typedef itk::ImageFileReader< FloatImageType2D >   FileFloat2DReaderType;
  typedef itk::ImageSeriesReader< FloatImageType >   SeriesReaderType;
  typedef itk::ImageFileReader< VectorImageType >    FileVectorReaderType;

  // Readers IO
  typedef itk::MetaImageIO MetaImageIOType;
  typedef itk::JPEGImageIO JPEGImageIOType;
  typedef itk::TIFFImageIO TIFFImageIOType;
  typedef itk::PNGImageIO PNGImageIOType;
  typedef itk::GDCMImageIO DicomImageIOType;
  typedef itk::DICOMImageIO2 DicomSeriesImageIOType;
  typedef itk::RawImageIO<InputPixelType,Dimension>  RawReaderType;
  typedef itk::RawImageIO<InputPixelType,Dimension>  RawWriterType;

  typedef itk::RawImageIO<InputPixelType,Dimension>::IOComponentType PixelType;    
  // Writers
  typedef itk::ImageFileWriter< UCharImageType >     UCharWriterType;
  typedef itk::ImageFileWriter< FloatImageType2D >   FloatWriter2DType;
  typedef itk::ImageFileWriter< FloatImageType >     FloatWriterType;

  // Viewers 
  typedef fltk::VTKImageViewer<InputImageType::PixelType>  VTKImageViewerType;
  //typedef  GLSliceView< ImagePixelType, OverlayPixelType > GLSliceViewType;
  typedef fltk::MyImageViewer<InputPixelType,InputPixelType>     MyViewerType;
  typedef ImageViewer<FloatPixelType,InputPixelType>             ViewerTypeFloat;
  typedef ImageViewer<InputPixelType,InputPixelType>             ViewerType;
  // Casters
  typedef itk::CastImageFilter<UCharImageType,FloatImageType> CastCharToRealFilterType;
  typedef itk::CastImageFilter<FloatImageType,UCharImageType> CastRealToCharFilterType;
  typedef itk::CastImageFilter<InputImageType2D,InputImageType> Cast2Dto3DFilterType;
  typedef itk::CastImageFilter<InputImageType,InputImageType2D> Cast3Dto2DFilterType;
  // Filtros:
  typedef itk::GradientMagnitudeImageFilter< InputImageType,InputImageType  >  GradientFilterType;
  //typedef itk::SimpleFuzzyConnectednessScalarImageFilter< InputImageType,InputImageType  >  FuzzyConnectFilterType;
  //typedef itk::AnisotropicDiffusionImageFilter<InputImageType,InputImageType  >  AnisotropicDiffusionType; 
  typedef itk::AnisotropicDiffusionImageFilter<InputImageType,InputImageType>::TimeStepType MyTimeStepType;
  typedef itk::ThresholdSegmentationLevelSetImageFilter<FloatImageType2D,FloatImageType2D> SegmentationFilterType;
  typedef itk::FastMarchingImageFilter<FloatImageType2D, FloatImageType2D >   FastMarchingFilterType;
  typedef itk::BilateralImageFilter<InputImageType,InputImageType  >  BilateralFilterType;
  typedef itk::GradientAnisotropicDiffusionImageFilter<InputImageType,InputImageType  >  GradientDiffusionType;
  typedef itk::CurvatureAnisotropicDiffusionImageFilter<FloatImageType,FloatImageType  >  CurvatureDiffusionType;
  typedef itk::CurvatureAnisotropicDiffusionImageFilter<InputImageType2D,InputImageType2D  >  Curvature2DDiffusionType;
  typedef itk::BinaryThresholdImageFilter<FloatImageType2D,InputImageType2D >   BinaryThresholdType;
  typedef itk::BinaryThresholdImageFilter<InputImageType, InputImageType> BinaryThresholdFilterType;
  typedef itk::RecursiveGaussianImageFilter<InputImageType, InputImageType > GaussianFilterType;
  typedef itk::MedianImageFilter<InputImageType, InputImageType > MedianFilterType;
  typedef itk::MeanImageFilter<InputImageType, InputImageType > MeanFilterType;
  typedef itk::WatershedImageFilter<InputImageType> WatershedFilterType;
  typedef itk::ChangeLabelImageFilter<InputImageType,InputImageType> RelabelFilterType;
  typedef itk::RescaleIntensityImageFilter<InputImageType,InputImageType> RescaleFilterType;
  typedef itk::ZeroCrossingBasedEdgeDetectionImageFilter<InputImageType,InputImageType> ZeroEdgeFilterType;
  
  typedef itk::RelabelComponentImageFilter<InputImageType,InputImageType> RelabelCompFilterType;
  typedef itk::AddImageFilter<InputImageType,InputImageType,InputImageType> AddFilterType;
  typedef itk::MultiplyImageFilter<InputImageType,InputImageType,InputImageType> MultiplyFilterType;
  typedef itk::AbsImageFilter< InputImageType, InputImageType> ABSFilterType;
  typedef itk::AbsoluteValueDifferenceImageFilter< InputImageType, InputImageType, InputImageType> ABSValDifFilterType;
  typedef itk::InvertIntensityImageFilter< InputImageType, InputImageType> InvertFilterType;
  // Structured Element 
  typedef itk::BinaryBallStructuringElement< InputPixelType, Dimension> StructuringElementType;
  typedef itk::ErodeObjectMorphologyImageFilter<InputImageType, InputImageType, StructuringElementType > ErodeFilterType;
  typedef itk::DilateObjectMorphologyImageFilter<InputImageType, InputImageType, StructuringElementType > DilateFilterType;
  // Iterators
  typedef itk::ImageRegionConstIterator<InputImageType> ConstIteratorType;
  typedef itk::ImageRegionIterator<InputImageType> IteratorType;
  // Filtros propios
  typedef itk::WienerFilter< InputImageType, InputImageType >  WienerFilterType;
  typedef itk::ASRFilter< InputImageType, InputImageType >     ASRFilterType;
  typedef itk::ASRFilter< InputImageType2D, InputImageType2D > ASRFilterType2D;
  typedef itk::SRADFilter< InputImageType, InputImageType >    SRADFilterType;
  typedef SRADFilterType::TensorFilterType                     SRADDiffusionType;
  typedef itk::DPADFilter< InputImageType, InputImageType >    DPADFilterType;
  //typedef DPADFilterType::TensorFilterType                     DPADDiffusionType;

  typedef itk::MLD< InputImageType, InputImageType > RegisterMLDType;
  // Connector VTK
  typedef itk::ImageToVTKImageFilter< InputImageType > ConnectorType;  
  typedef itk::VTKImageExport< InputImageType> VTKexportType;
  // Para KNN:
  typedef itk::Image<int, 1> VoronoiInputType;
  typedef VoronoiInputType::IndexType IndexType;
  typedef itk::voronoiFilter<VoronoiInputType, VoronoiInputType::PixelType, 1> VoronoiFilterType;
  typedef itk::Image< std::vector<int> , 1>  DiagramImageType; 
  typedef itk::classifyKNNCore<InputImageType, DiagramImageType, InputImageType, 3> classifyKNNCoreType;  

  // Filtro de Canny
  typedef itk::CannyEdgeDetectionImageFilter< InputImageType, InputImageType > CannyFilterType;


  virtual void Load(void){};
  virtual void LoadME(void){};
  virtual void LoadKNN(void){};
  virtual void LoadRaw(void){};
  virtual void LoadRaw(const char* filename){};
  virtual void LoadMeta(void){};
  virtual void Save(void);
  virtual void Save(const char * outputFileName);
  virtual void SaveFloat(void);
  virtual void SaveFloat(const char * outputFileName);

  virtual void SetDimensionX( unsigned int );
  virtual void SetDimensionY( unsigned int );
  virtual void SetDimensionZ( unsigned int );
  virtual void SetSpacingX( double );
  virtual void SetSpacingY( double );
  virtual void SetSpacingZ( double );

  virtual void SetNIterations( unsigned int );
  virtual void SetConductance( double );
  virtual void SetTimeStep( MyTimeStepType );

  //  virtual void SetThresholdFuzzy( double ){};
  virtual void SetInMean( double );
  virtual void SetInVar( double );
  virtual void SetInDifMean( double );
  virtual void SetInDifVar( double );
  virtual void SetInWeight( double );

  virtual void Show(void){};
  virtual void ShowImageSrc(int window){};
  virtual void ShowImageOut(void);
  virtual void ShowVTK(void);

  virtual void Distribuir(InputImageType::Pointer aux, int window){};

  virtual void GradientDiffusionFilter(void){};
  virtual void ThresholdLevelSetSegmentation(void){};

  virtual void Quit(void){};
  
  unsigned int m_activeinput;

  UsimagToolBase();
  virtual ~UsimagToolBase();
  
  const char * m_filename;
  const char * ME_filename;
  const char * KNN_filename;

protected:
  
  // Filtros
  GradientDiffusionType::Pointer     m_GradientDiffusionFilter;
  GradientFilterType::Pointer        m_GradientFilter;
  MeanFilterType::Pointer            m_MeanFilter;
  SegmentationFilterType::Pointer    m_LevelSetSegmentation; 
  //  FuzzyConnectFilterType::Pointer    m_FuzzyConnectSeg;
  FastMarchingFilterType::Pointer    m_fastMarching;
  CurvatureDiffusionType::Pointer    m_Curvature;
  Curvature2DDiffusionType::Pointer  m_Curvature2D;
  BilateralFilterType::Pointer       m_BilateralFilter;
  BinaryThresholdType::Pointer       m_thresholder;     
  GaussianFilterType::Pointer        m_GaussianFilter;
  MedianFilterType::Pointer          m_MedianFilter;
  WatershedFilterType::Pointer       m_WatershedFilter;
  RelabelFilterType::Pointer         m_RelabelFilter;
  RescaleFilterType::Pointer         m_RescaleFilter;
  ABSFilterType::Pointer             m_ABSFilter;
  ABSValDifFilterType::Pointer       m_ABSValDifFilter;
  InvertFilterType::Pointer	     m_InvertFilter;
  ZeroEdgeFilterType::Pointer	     m_ZeroEdgeFilter;

  RelabelCompFilterType::Pointer     m_RelabelCompFilter;
  BinaryThresholdFilterType::Pointer m_ThresholdFilter;
  AddFilterType::Pointer             m_AddFilter;
  MultiplyFilterType::Pointer        m_MultiplyFilter;
  WienerFilterType::Pointer          m_WienerFilter;
  ASRFilterType::Pointer             m_ASRFilter;
  ASRFilterType2D::Pointer           m_ASRFilter2D;
  SRADFilterType::Pointer            m_SRADFilter;
  SRADDiffusionType::Pointer         m_SRADdiffusion;
  DPADFilterType::Pointer            m_DPADFilter;
  //DPADDiffusionType::Pointer         m_DPADdiffusion;

  RegisterMLDType::Pointer           m_regMLD;

  ErodeFilterType::Pointer           m_ErodeFilter; 
  DilateFilterType::Pointer          m_DilateFilter;
  StructuringElementType             m_structuringElement;

  classifyKNNCoreType::Pointer       m_KnnFilter; 

  CannyFilterType::Pointer			 m_CannyFilter;

  // Readers 
  FileReaderType::Pointer     m_MetaImageReader;
  FileVectorReaderType::Pointer  m_MetaVectorImageReader;
  FileReaderType::Pointer     m_RawImageReader;
  FileReaderType::Pointer     m_JPEGImageReader; 
  FileReaderType::Pointer     m_VTKImageReader; 
  FileReaderType::Pointer     m_TIFFImageReader;
  FileReaderType::Pointer     m_PNGImageReader;
  FileFloatReaderType::Pointer  m_DicomImageReader;
  FileReaderType::Pointer     m_GenericReader;
  SeriesReaderType::Pointer   m_DicomSeriesImageReader;
  FileFloat2DReaderType::Pointer  m_JPEGFloatImageReader;

  MetaImageIOType::Pointer    m_MetaReader; 
  RawReaderType::Pointer      m_RawReader; 
  JPEGImageIOType::Pointer    m_JPEGReader;
  TIFFImageIOType::Pointer    m_TIFFReader;
  PNGImageIOType::Pointer     m_PNGReader;
  DicomImageIOType::Pointer   m_DicomReader;
  DicomSeriesImageIOType::Pointer  m_DicomSeriesReader;
  itk::DICOMSeriesFileNames::Pointer m_nameGenerator;
 
  // Writers 
  FloatWriterType::Pointer    m_MetaImageWriter;
  FloatWriterType::Pointer    m_FloatImageWriter;
  FloatWriterType::Pointer    m_ImageWriter; 
  RawReaderType::Pointer      m_RawWriter; 

  MetaImageIOType::Pointer    m_MetaWriter;

  // Imagenes de entrada 
  InputImageType::Pointer     m_InputImage[4];
  InputRegionType             m_RegionInit; 
  SizeType                    m_Size;
  FloatImageType2D::Pointer   m_FloatInputImage;

  VectorImageType::Pointer     m_VectorImage;

  // Imagenes de salida e intermedias
  FloatImageType::Pointer     m_FloatImage;
  FloatImageType::Pointer     m_FloatImage2;   
  InputImageType::Pointer     m_OutputImage;
  
  // Visores 
  ViewerType                  m_ViewerOut;
  ViewerTypeFloat             m_ViewerFloat;
  ViewerTypeFloat             m_ViewerFloat2;
  // PARA VTK 
  VTKImageViewerType::Pointer m_VTKviewer;
  ConnectorType::Pointer      m_connector;

  vtkMarcacionElipsoide* m_vtkMarcacionElipsoide;
  vtkPlantillaAjustada* m_vtkPlantillaAjustada;
  vtkFuncionVerosimilitud* m_vtkFuncionVerosimilitud;
  vtkOptimizaContorno* m_vtkOptimizaContorno;
  vtkValidacion* m_vtkValidacion;

  VTKexportType::Pointer m_VTKexporter; 
  // casters
  CastCharToRealFilterType::Pointer m_char_to_real;
  CastRealToCharFilterType::Pointer m_real_to_char;
  Cast2Dto3DFilterType::Pointer     m_2Dto3DCaster;
  Cast3Dto2DFilterType::Pointer     m_3Dto2DCaster;
  Cast2Dto3DFilterType::Pointer     m_SegmentedCaster;
  Cast3Dto2DFilterType::Pointer     m_InputCaster;
  Cast2Dto3DFilterType::Pointer     m_OutputCaster;
  
  int                         m_FileType;
  PixelType                   m_PixelType;
  unsigned int                m_ByteOrder;
  unsigned int                m_NImagesCargadas;
  unsigned int                m_NumberOfPixelsInX;
  unsigned int                m_NumberOfPixelsInY;
  unsigned int                m_NumberOfPixelsInZ;
 
  // Parametros
  unsigned int                m_NIterations;
  MyTimeStepType              m_TimeStep;
  double                      m_Conductance;
  float m_threshold;
  double m_inmean, m_invar, m_indifmean, m_indifvar, m_inweight;
private:
  
  ViewerType                  m_ViewerSrc1;
  ViewerType                  m_ViewerSrc2;
  ViewerType                  m_ViewerSrc3;
   
  double m_SpacingX;
  double m_SpacingY;
  double m_SpacingZ;
  
};



