/*=========================================================================

  Program:   UsimagToolConsole
  Language:  C++
  Date:      5-07-2007
  Version:   1.0

  Copyright (c) 2007 Laboratoy of Image Processing, UVA. All rights reserved.
  See http://www.lpi.tel.uva.es/UsimagTool for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE. 

=========================================================================*/

#include "UsimagToolConsole.h"
#include "GenericImageToImageFilter.h"
#include <itkMinimumMaximumImageCalculator.h>
#include <itkEventObject.h>
#include <itkSpatialObjectReader.h>
#include <itkLandmarkSpatialObject.h>
#include "vtkITKUtility.h"
#include "vtkPolyDataWriter.h"
#include <iomanip>
#include <string>

UsimagToolConsole::UsimagToolConsole()
{
  //progressSlider->Observe( m_MetaImageReader );


  for (int i=0; i < 4; i++ ) {
    ImageViewer[i]->SetImage(m_InputImage[i]);
    ImageViewer[i]->ClickSelectCallBack( ClickSelectCallback, (void *)this );
    ImageViewer[i]->Update();
  }

  m_InFilePageFormat->add("MetaHeader",0,NULL,NULL,0);
  m_InFilePageFormat->add("VOL",0,NULL,NULL,0);
  m_InFilePageFormat->add("Raw",0,NULL,NULL,0);
  m_InFilePageFormat->add("JPEG",0,NULL,NULL,0);
  m_InFilePageFormat->add("PNG",0,NULL,NULL,0);
  m_InFilePageFormat->add("TIFF",0,NULL,NULL,0);
  m_InFilePageFormat->add("DICOM",0,NULL,NULL,0);
  m_InFilePageFormat->add("DICOM Series",0,NULL,NULL,0);
  m_InFilePageFormat->add("VTK",0,NULL,NULL,0);

  m_FileFormatPattern[0] = "mha,mhd";
  m_FileFormatPattern[1] = "VOL";
  m_FileFormatPattern[2] = "raw*";
  m_FileFormatPattern[3] = "jpg,jpeg";
  m_FileFormatPattern[4] = "png";
  m_FileFormatPattern[5] = "tif,tiff";
  m_FileFormatPattern[6] = "dcm"; 
  m_FileFormatPattern[7] = "vtk";

  //m_FileFormatPattern[7] = "gipl,gipl.gz";
  //m_FileFormatPattern[8] = "vtk";
  //m_FileFormatPattern[9] = "ima";

  m_InByteOrder->add("Little Endian",0,NULL,NULL,0);
  m_InByteOrder->add("Big Endian",0,NULL,NULL,0);
  
  m_InPixelType->add("UCHAR",0,NULL,NULL,0);
  m_InPixelType->add("CHAR",0,NULL,NULL,0);
  m_InPixelType->add("USHORT",0,NULL,NULL,0);
  m_InPixelType->add("SHORT",0,NULL,NULL,0);  
  m_InPixelType->add("FLOAT",0,NULL,NULL,0);
  m_InPixelType->add("DOUBLE",0,NULL,NULL,0);
  
  m_destino->add("Output",0,NULL,NULL,0);
  m_destino->add("Imagen 1",0,NULL,NULL,0);
  m_destino->add("Imagen 2",0,NULL,NULL,0);
  m_destino->add("Imagen 3",0,NULL,NULL,0);
  m_destino->add("Imagen 4",0,NULL,NULL,0);

  m_op1->add("Output",0,NULL,NULL,0);
  m_op1->add("Imagen 1",0,NULL,NULL,0);
  m_op1->add("Imagen 2",0,NULL,NULL,0);
  m_op1->add("Imagen 3",0,NULL,NULL,0);
  m_op1->add("Imagen 4",0,NULL,NULL,0);

  m_op2->add("Output",0,NULL,NULL,0);
  m_op2->add("Imagen 1",0,NULL,NULL,0);
  m_op2->add("Imagen 2",0,NULL,NULL,0);
  m_op2->add("Imagen 3",0,NULL,NULL,0);
  m_op2->add("Imagen 4",0,NULL,NULL,0);

  m_tipoProcesado->add("    Basic Operations",0,NULL,NULL,0);
  m_tipoProcesado->add("    Filtering",0,NULL,NULL,0);
  m_tipoProcesado->add("    Segmentation",0,NULL,NULL,0);
  m_tipoProcesado->add("    Registration",0,NULL,NULL,0);

  m_tipo_filtroWiener->add("Homogeneous",0,NULL,NULL,0);
  m_tipo_filtroWiener->add("Anisotropic",0,NULL,NULL,0);
  m_tipo_filtroWiener->add("Mixed",0,NULL,NULL,0);

  m_seeds = NodeContainer::New();
  m_destino->value(1);
  
  // Para actualizar resultado de Level Set en cada iteracion 
  typedef itk::SimpleMemberCommand< UsimagToolConsole > SimpleCommandType;

  SimpleCommandType::Pointer iterationCommand = SimpleCommandType::New();
  iterationCommand->SetCallbackFunction( this,
      & UsimagToolConsole::UpdateViewerAfterIteration );
  m_LevelSetSegmentation->AddObserver( itk::IterationEvent(), iterationCommand);
  ////////////////

  num_points->value(0);

  // Parametros por defecto de Level Set
  RMSError->value(0.02);
  curvature->value(1);
  propagation->value(1);
  edge->value(1);
  isovalue->value(1);
  lowerThreshold->value(0);
  upperThreshold->value(100);
  reverseExpansionDir->value(0);
  maxCurvatureTimeStep->value(0.25);
  maxPropagationTimeStep->value(0.25);
  diffusionIterations->value(0);
  diffusionConductance->value(1);
  diffusionTimeStep->value(0.125);
  threshIterations->value(0);
  threshConductance->value(0.5);
  threshTimeStep->value(0.1);
}

UsimagToolConsole::~UsimagToolConsole()
{

}


void UsimagToolConsole::Quit() {
  panel->hide();
  m_ViewerOut.Hide();
}

void UsimagToolConsole::ClickSelectCallback(float x, float y, float z,
                                           float v, int c, void * args )
{
  UsimagToolConsole* self = static_cast<UsimagToolConsole *>( args );
  self->GetClickPoints(x, y);
}

void UsimagToolConsole::OnFileInputChange() {
  const char *text = m_InFilePageBrowser->value();
  if (text != NULL && strlen(text) > 0) {    
    m_filename = text;
  }

  if (text != NULL && strlen(text) > 0) {
    int v = DetermineFileFormatFromFileName(text);    
    if (v != -1) {
      m_InFilePageFormat->value(v+1);
      m_FileType = m_InFilePageFormat->value();
    }
  }

}

int UsimagToolConsole::DetermineFileFormatFromFileName(const char *testFile)
{
  
  // Iterate over the known file types
  for(int i = 0;i < m_FORMAT_COUNT;i++) {
    // Create a matching pattern
    std::string pattern = "*.{" + m_FileFormatPattern[i] + "}";
    
    // Check if the filename matches the pattern
    if(fl_filename_match(testFile,pattern.c_str()))
      return i;
    
  }

  // Failed: return illegal pattern
  return -1;
}

void UsimagToolConsole::OnFileFormatChange()
{
  m_FileType = m_InFilePageFormat->value();
}

void UsimagToolConsole::OnByteOrderChange()
{
  m_ByteOrder = m_InByteOrder->value();
}


//*** Vero
void UsimagToolConsole::OnTipoProcesadoChange()
{
	switch(m_tipoProcesado->value()){
		case 0: // Basic Operations
			m_Filtering->hide();
			m_Segmentation->hide();
			m_Registration->hide();
			m_BasicOperations->show();
			break;
		case 1: // Filtering
			m_BasicOperations->hide();
			m_Segmentation->hide();
			m_Registration->hide();
			m_Filtering->show();
			break;
		case 2: // Segmentation
			m_BasicOperations->hide();
			m_Registration->hide();
			m_Filtering->hide();
			m_Segmentation->show();
			break;
		case 3:	// Registration
			m_BasicOperations->hide();
			m_Segmentation->hide();
			m_Filtering->hide();
			m_Registration->show();
			break;
	}
}
//*** Vero

void UsimagToolConsole::OnPixelTypeChange()
{
  switch(m_InPixelType->value()) { 
  case 0: 
    return;
    break;
  case 1:
    m_PixelType = itk::ImageIOBase::UCHAR;
    break;
  case 2:
    m_PixelType = itk::ImageIOBase::CHAR;
    break;
  case 3:
    m_PixelType = itk::ImageIOBase::USHORT;
    break;
  case 4:
    m_PixelType = itk::ImageIOBase::SHORT;
    break;
  case 5:
    m_PixelType = itk::ImageIOBase::FLOAT;
    break;
  case 6:
    m_PixelType = itk::ImageIOBase::DOUBLE;
    break;
  default:
    break;
  }
}

void UsimagToolConsole::OnTipoFiltradoChange()
{
  m_tipofiltrado = m_filtrado->value();
}

void UsimagToolConsole::OnTipoFiltroWienerChange()
{
  m_filtroWiener = m_tipo_filtroWiener->value();
}


void UsimagToolConsole::Abrir( int filetype )
{
  //std::cout << "abriendo; filetype: "  << filetype << std::endl;

  switch(m_FileType) { 
  case 0: 
    return;
    break;
  case 1:
    LoadMeta();
    break;
  case 2:
    LoadVOL();
    break;
  case 3:
    LoadRaw();
    break;
  case 4:
    LoadJPEG();
    break;
  case 5:
    LoadPNG();
    break;
  case 6:
    LoadTIFF();
    break;
  case 7:
    LoadDicom();
    break;
  case 8:
    LoadDicomSeries();
    break;
  case 9:
    LoadVTK();
    break;
  default:
    break;
  }
  
}

void UsimagToolConsole::Load( void )
{
  m_filename = fl_file_chooser("Image Filename","*.{raw,mhd,mha,VOL,jpeg,jpg,tif,tiff,png}","");

  m_InFilePageBrowser->value(m_filename);
  if( !m_filename ) {
    return;
  }
}

void UsimagToolConsole::LoadME( void )
{
  std::cout << "En LoadME" << std::endl;
  ME_filename = fl_file_chooser("Image Filename","*.{vtk,txt}","");

  ME_FilePageBrowser->value(ME_filename);
  if( !ME_filename ) {
    return;
  }
}

void UsimagToolConsole::LoadKNN( void )
{
  std::cout << "En LoadKNN" << std::endl;
  KNN_filename = fl_file_chooser("Points Filename","*.{txt,mhd}","");

  KNN_FilePageBrowser->value(KNN_filename);
  if( !KNN_filename ) {
    return;
  }
}

void UsimagToolConsole::OnProgress(itk::Object *object, const itk::EventObject &event)
{
  // Get the value of the progress
  itk::ProcessObject *po = reinterpret_cast<ProcessObject *>(object);
  float progress = po->GetProgress();

  //std::cout << "progress "  << progress << std::endl;
  
  float max = 1.0;
  if ( m_InputImage[m_activeinput]->GetLargestPossibleRegion().GetSize()[2] == 1) {
    max = 2.0;
  }
  m_ProgressCounter->value(100*progress/max);

  // Display the filter's progress
  progressSlider->value(progress/max);

  // Let the UI refresh
  Fl::check();
}

void UsimagToolConsole::LoadMeta( void )
{
  if (m_PixelComponents->value() == 1) {

  m_MetaImageReader->SetFileName(m_filename);
  
  // Attempt to read
  try {
    m_MetaImageReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_MetaImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

  m_NImagesCargadas++;

  } else if (m_PixelComponents->value() == 2) {
   
    m_MetaVectorImageReader->SetFileName(m_filename);
    // Attempt to read
    try {
      m_MetaVectorImageReader->Update();
    } catch( itk::ExceptionObject & e ) {
      fl_alert( e.GetDescription() );
    }

    //std::cout << "Vector Image: " << m_MetaVectorImageReader->GetOutput()->PrintSelf() << std::endl;
    VectorImageType::Pointer aux = VectorImageType::New();
  }

}

void UsimagToolConsole::LoadVOL( void )
{

  m_GenericReader->SetFileName(m_filename);
  // Attempt to read
  try {
    m_GenericReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_GenericReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

  m_NImagesCargadas++;

}

void UsimagToolConsole::LoadRaw( void )
{
   panelDim->show();
}

void UsimagToolConsole::LoadRaw( const char * filename ) 
{
  m_RawImageReader->SetFileName( filename );
  m_RawReader->SetFileDimensionality( 3 );
  m_RawReader->SetDimensions( 0, (unsigned int)m_NumberOfPixelsInX->value() );
  m_RawReader->SetDimensions( 1, (unsigned int)m_NumberOfPixelsInY->value() );
  m_RawReader->SetDimensions( 2, (unsigned int)m_NumberOfPixelsInZ->value() );
  m_RawReader->SetComponentType(m_PixelType);
  if (m_ByteOrder == 1) {
    m_RawReader->SetByteOrderToLittleEndian();
  } else {
    m_RawReader->SetByteOrderToBigEndian();
  }
  // Attempt to read
  try {
    m_RawImageReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }
   
  InputImageType::Pointer aux = InputImageType::New();
  aux = m_RawImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

  m_NImagesCargadas++;

}

void UsimagToolConsole::LoadJPEG(  )
{

  m_JPEGImageReader->SetFileName( m_filename );
  try {
    m_JPEGImageReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_JPEGImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

}

void UsimagToolConsole::LoadVTK(  )
{

  m_VTKImageReader->SetFileName( m_filename );
  try {
    m_VTKImageReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_VTKImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

}

//void UsimagToolConsole::LoadJPEG(  )
//{

//  m_JPEGFloatImageReader->SetFileName( m_filename );

  // Attempt to read
//  try {
//    m_JPEGFloatImageReader->Update();
//  } catch( itk::ExceptionObject & e ) {
//    fl_alert( e.GetDescription() );
//  }

//  m_FloatInputImage = m_JPEGFloatImageReader->GetOutput();
//  m_InputCaster->SetInput(m_FloatInputImage);
//  m_InputCaster->Update();
//  m_ViewerFloat.SetImage(m_InputCaster->GetOutput());
//  m_ViewerFloat.Show();
//  m_ViewerFloat.ViewDetails(false);
//  m_ViewerFloat.ViewValue(false);

//}

void UsimagToolConsole::LoadTIFF(  )
{

  m_TIFFImageReader->SetFileName( m_filename );  

  // Attempt to read
  try {
    m_TIFFImageReader->Update();
  } catch( itk::ExceptionObject & e ) {
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_TIFFImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  

}

void UsimagToolConsole::LoadPNG(  )
{

  m_PNGImageReader->SetFileName( m_filename );

  // Attempt to read
  try {
    m_PNGImageReader->Update();
  } catch( itk::ExceptionObject & e ){
    fl_alert( e.GetDescription() );
  }

  InputImageType::Pointer aux = InputImageType::New();
  aux = m_PNGImageReader->GetOutput();
  this->Distribuir(aux,m_activeinput);  
}

void UsimagToolConsole::LoadDicom(  )
{

  m_DicomImageReader->SetFileName( m_filename );
  try {
    m_DicomImageReader->Update();
  } catch( itk::ExceptionObject & e ){
    fl_alert( e.GetDescription() );
  }

  m_ViewerFloat.SetImage(m_DicomImageReader->GetOutput());
  m_ViewerFloat.Show();
  //InputImageType::Pointer aux = InputImageType::New();
  //m_real_to_char->SetInput(m_DicomImageReader->GetOutput());
  //aux = m_real_to_char->GetOutput();
  //m_real_to_char->Update();
  //this->Distribuir(aux,m_activeinput);  
}

void UsimagToolConsole::directorio(const char* file,char* dir) {
  char barra[2] = "/";
  int diferencia;
  diferencia = strlen(file);
  if (strrchr(file,'/') != NULL) {
    diferencia -= strlen(strrchr(file,'/'));
  /* strlcpy(dir,file,diferencia+1);*/
  strncpy(dir,file,diferencia+1);
  dir[diferencia] = '\0';
  } else {
    dir[0] = '.';
    dir[1] = '\0';
  }
}

void UsimagToolConsole::LoadDicomSeries( ) {
    // Falta poner: m_directory, y m_series_name (opcional)
    char m_directory[300];
    directorio(m_filename,m_directory);
    char *m_series_name;
    int series_name = 0;  
    
    std::cout << std::endl << "The directory: " << std::endl;
    std::cout << std::endl << m_directory << std::endl << std::endl;
    std::cout << "Contains the following DICOM Series: ";
    std::cout << std::endl << std::endl;

    m_nameGenerator->SetDirectory( m_directory );
    typedef std::vector<std::string> seriesIdContainer;
    const seriesIdContainer & seriesUID = m_nameGenerator->GetSeriesUIDs();

    seriesIdContainer::const_iterator seriesItr = seriesUID.begin();
    seriesIdContainer::const_iterator seriesEnd = seriesUID.end();
  
    while( seriesItr != seriesEnd ) {
      std::cout << seriesItr->c_str() << std::endl;
      seriesItr++;
    }
  
    std::cout << std::endl << std::endl;
    std::cout << "Now reading series: " << std::endl << std::endl;

    typedef std::vector<std::string> fileNamesContainer;
    fileNamesContainer fileNames;

    if( series_name == 0 ) { // If no optional third argument 
      std::cout << seriesUID.begin()->c_str() << std::endl;
      fileNames = m_nameGenerator->GetFileNames();
    } else {
      //fileNames = m_nameGenerator->GetFileNames( m_series_name );
      //std::cout << m_series_name << std::endl;
    }
    std::cout << std::endl << std::endl;

    m_DicomSeriesImageReader->SetFileNames( fileNames );    

    try {
      m_DicomSeriesImageReader->Update();
    } catch (itk::ExceptionObject &ex) {
      std::cout << ex << std::endl;
    }

    m_ViewerFloat.SetImage(m_DicomSeriesImageReader->GetOutput());
    m_ViewerFloat.Show();
    //InputImageType::Pointer aux = InputImageType::New();
    //aux = m_DicomSeriesImageReader->GetOutput();
    //this->Distribuir(aux,m_activeinput);
}

void UsimagToolConsole::Distribuir( InputImageType::Pointer aux, int window ) {  
  ConstIteratorType auxIt(aux,aux->GetLargestPossibleRegion());
   
  m_InputImage[window]->SetRegions(aux->GetLargestPossibleRegion());
  m_InputImage[window]->SetSpacing(aux->GetSpacing());
  m_InputImage[window]->Allocate();
  if (window == 0) {    
    // Copiamos la imagen aux en imagen InputImage1
    IteratorType imageIt1(m_InputImage[0],m_InputImage[0]->GetRequestedRegion());
    for (auxIt.GoToBegin(),imageIt1.GoToBegin();!auxIt.IsAtEnd();++auxIt, ++imageIt1) {
      imageIt1.Set(auxIt.Get());
    }
  } else if (window == 1) {
    // Copiamos la imagen aux en imagen InputImage2
    IteratorType imageIt2(m_InputImage[1],m_InputImage[1]->GetRequestedRegion());
    for (auxIt.GoToBegin(),imageIt2.GoToBegin();!auxIt.IsAtEnd();++auxIt, ++imageIt2) {
      imageIt2.Set(auxIt.Get());
    }
  
  } else if (window == 2) {
    // Copiamos la imagen aux en imagen InputImage3
    IteratorType imageIt3(m_InputImage[2],m_InputImage[2]->GetRequestedRegion());
    for (auxIt.GoToBegin(),imageIt3.GoToBegin();!auxIt.IsAtEnd();++auxIt, ++imageIt3) {
      imageIt3.Set(auxIt.Get());
    }
  } else {
    // Copiamos la imagen aux en imagen InputImage4
    IteratorType imageIt4(m_InputImage[3],m_InputImage[3]->GetRequestedRegion());
    for (auxIt.GoToBegin(),imageIt4.GoToBegin();!auxIt.IsAtEnd();++auxIt, ++imageIt4) {
      imageIt4.Set(auxIt.Get());
    }
   
  }
  ShowImageSrc(m_activeinput);
  
}

void UsimagToolConsole::ShowImageSrc(int window)
{
  ImageViewer[window]->SetImage( m_InputImage[window]);
  ImageViewer[window]->Show();
  ImageViewer[window]->ViewDetails(false);
  ImageViewer[window]->ViewValue(false);
  
  float iwDiff  = ImageViewer[window]->iwMax() - ImageViewer[window]->iwMin();
  float b       = (float)((int)log10(iwDiff)-2);
  double iwMin  = ((int)(ImageViewer[window]->iwMin()*pow((float)10, (float)-b)))/pow((float)10,(float)-b);
  double iwMax  = ((int)(ImageViewer[window]->iwMax()*pow((float)10, (float)-b)))/pow((float)10,(float)-b);
  double iwStep = (iwMax-iwMin)/100.0;
  sliceNumberSlider[window]->range( 0.0f, ImageViewer[window]->numSlices() );
  //intensityWindowingMinSlider->range(iwMin-iwStep,iwMax+iwStep);
  //intensityWindowingMaxSlider->range(iwMin-iwStep,iwMax+iwStep);
  sliceNumberSlider[window]->value((float)ImageViewer[window]->sliceNum());
  //intensityWindowingMinSlider->step(iwStep);
  //intensityWindowingMaxSlider->step(iwStep);
  //intensityWindowingMinSlider->value(iwMin);
  //intensityWindowingMaxSlider->value(iwMax);
}

void UsimagToolConsole::OnSlice0Change(unsigned int value) {

  ImageViewer[0]->SelectSlice(value);
  if ( m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[0]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[1]->SelectSlice(value);
    sliceNumberSlider[1]->value(value);
  }
  if ( m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[0]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[2]->SelectSlice(value);
    sliceNumberSlider[2]->value(value);
  }
}

void UsimagToolConsole::OnSlice1Change(unsigned int value) {
  ImageViewer[1]->SelectSlice(value);
  if ( m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[0]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[0]->SelectSlice(value);
    sliceNumberSlider[0]->value(value);
  }
  if ( m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[2]->SelectSlice(value);
    sliceNumberSlider[2]->value(value);
  }
}

void UsimagToolConsole::OnSlice2Change(unsigned int value) {
  ImageViewer[2]->SelectSlice(value);
   if ( m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[1]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[1]->SelectSlice(value);
    sliceNumberSlider[1]->value(value);
  }
  if ( m_InputImage[0]->GetLargestPossibleRegion().GetSize()[2] > 1 && 
       m_InputImage[0]->GetLargestPossibleRegion().GetSize()[2] == 
       m_InputImage[2]->GetLargestPossibleRegion().GetSize()[2]) {
    ImageViewer[0]->SelectSlice(value);
    sliceNumberSlider[0]->value(value);
  }
}


void UsimagToolConsole::OnSlice3Change(unsigned int value) {
  ImageViewer[3]->SelectSlice(value);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
///              AQUI EMPIEZAN LOS FILTROS              /////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////



void UsimagToolConsole::ResetAllParameters() {

  m_thresholder->SetOutsideValue(0);
  m_thresholder->SetInsideValue(1);
  m_thresholder->SetUpperThreshold(0.5);
  m_thresholder->SetLowerThreshold(-0.5);
  
  //m_Curvature->SetNumberOfIterations(static_cast<unsigned int>(diffusionIterations->value()));
  //m_Curvature->SetConductanceParameter(diffusionConductance->value());
  //m_Curvature->SetTimeStep(diffusionTimeStep->value());
  
  m_LevelSetSegmentation->AbortGenerateDataOff();
  m_LevelSetSegmentation->SetUpperThreshold( upperThreshold->value() );
  m_LevelSetSegmentation->SetLowerThreshold( lowerThreshold->value() );
  m_LevelSetSegmentation->SetCurvatureScaling( curvature->value() );
  m_LevelSetSegmentation->SetPropagationScaling( propagation->value() );
  m_LevelSetSegmentation->SetEdgeWeight( edge->value() );
  m_LevelSetSegmentation->SetMaximumRMSError( RMSError->value() );
  m_LevelSetSegmentation->SetNumberOfIterations( (unsigned int)levelsetiterations->value() );
  m_LevelSetSegmentation->SetMaximumCurvatureTimeStep( maxCurvatureTimeStep->value() );
  m_LevelSetSegmentation->SetMaximumPropagationTimeStep( maxPropagationTimeStep->value() );
  m_LevelSetSegmentation->SetSmoothingIterations( (unsigned int)threshIterations->value() );
  m_LevelSetSegmentation->SetSmoothingConductance( threshConductance->value() );
  m_LevelSetSegmentation->SetSmoothingTimeStep( threshTimeStep->value() );

  if(reverseExpansionDir->value()) {
    m_LevelSetSegmentation->ReverseExpansionDirectionOn();
  }
  else {
    m_LevelSetSegmentation->ReverseExpansionDirectionOff();
  }
}


void UsimagToolConsole::ThresholdLevelSetSegmentation(void ) 
{
  std::cout << "Doing Level Set" << std::endl;
 
  // CommandPointer callback = CommandType::New();  
  // callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  //m_LevelSetSegmentation->AddObserver( itk::ProgressEvent(), callback);

  this->ResetAllParameters();

  m_InputCaster->SetInput(m_InputImage[m_activeinput]);
  m_InputCaster->Update();
  m_FloatInputImage = m_InputCaster->GetOutput();
  m_FloatInputImage->SetSpacing(1.0);

  m_LevelSetSegmentation->SetFeatureImage( m_FloatInputImage);
  m_LevelSetSegmentation->SetInput( m_fastMarching->GetOutput() );
  m_LevelSetSegmentation->SetIsoSurfaceValue(0.0);

  m_seedPosition[0] = (unsigned long)seedX->value();
  m_seedPosition[1] = (unsigned long)seedY->value();
  
  m_node.SetValue( -distance->value() );
  m_node.SetIndex( m_seedPosition );
  
  m_seeds->Initialize();
  m_seeds->InsertElement( 0, m_node );
  m_fastMarching->SetTrialPoints( m_seeds );
 
  m_SegmentedCaster->SetInput(m_thresholder->GetOutput());
  std::cout << "size : " << m_Curvature->GetOutput()->GetBufferedRegion().GetSize() << std::endl;
  m_fastMarching->SetOutputSize(m_FloatInputImage->GetBufferedRegion().GetSize() );
  
  //std::cout << "Iterations " << levelsetiterations->value() << std::endl;

  //m_FloatImage->SetRegions(m_FloatInputImage->GetLargestPossibleRegion());
  //m_FloatImage->Allocate();
  //m_FloatImage = m_LevelSetSegmentation->GetOutput();
  //m_FloatImage = const_cast<FloatImageType *>(m_LevelSetSegmentation->GetSpeedImage());
  
  try {
    m_LevelSetSegmentation->UpdateLargestPossibleRegion();    
  }
  catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
    std::cerr << e << std::endl;
  }

  //m_real_to_char->SetInput(m_FloatImage);
  //m_real_to_char->Update();
  //m_OutputImage = m_real_to_char->GetOutput();
  
  m_OutputCaster->SetInput(m_LevelSetSegmentation->GetOutput());
  m_OutputCaster->Update();
  m_OutputImage = m_OutputCaster->GetOutput();

  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::UpdateViewerAfterIteration()
{
  static unsigned int iterationCounter = 0;
  //std::cout << "UpdateViewerAfterIteration  " << "it n:" <<  m_LevelSetSegmentation->GetElapsedIterations() << std::endl;
  if( (iterationCounter%((int)updateIterations->value()) == 0)
      && (iterationCounter != 0) ) {
    // Move the pixel container and image information of the image we are working
    // on into a temporary image to  use as the input to the mini-pipeline.  This
    // avoids a complete copy of the image.
    FloatImageType2D::Pointer tmp = FloatImageType2D::New();
    tmp->SetRequestedRegion( m_LevelSetSegmentation->GetOutput()->GetRequestedRegion() );
    tmp->SetBufferedRegion( m_LevelSetSegmentation->GetOutput()->GetBufferedRegion() );
    tmp->SetLargestPossibleRegion( m_LevelSetSegmentation->GetOutput()->GetLargestPossibleRegion() );
    tmp->SetPixelContainer( m_LevelSetSegmentation->GetOutput()->GetPixelContainer() );
    tmp->CopyInformation( m_LevelSetSegmentation->GetOutput() );

    // update overlay
    m_thresholder->Modified();
    m_thresholder->SetInput(tmp);
    try
      {
      // Updates m_thresholder
      m_SegmentedCaster->UpdateLargestPossibleRegion();
      }
    catch( itk::ExceptionObject & excep )
      {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excep << std::endl;
      }

    // update the viewer with the current segmentation
    //ImageViewer[m_activeinput]->SetOverlay(m_SegmentedCaster->GetOutput());
    //ImageViewer[m_activeinput]->SetOverlayOpacity(1.0);
    //ImageViewer[m_activeinput]->Update();
    ImageViewer[m_activeinput]->SetOverlay(m_SegmentedCaster->GetOutput());
    ImageViewer[m_activeinput]->SetOverlayOpacity(1.0);
    ImageViewer[m_activeinput]->Update();
    Fl::check();

    elapsedIterations->value(m_LevelSetSegmentation->GetElapsedIterations());
    lastRMSChange->value(m_LevelSetSegmentation->GetRMSChange());
    progressSlider->value( m_LevelSetSegmentation->GetElapsedIterations()
			   / levelsetiterations->value() );
    m_ProgressCounter->value(100*m_LevelSetSegmentation->GetElapsedIterations()
			     / levelsetiterations->value());
  }
  iterationCounter++;
}

void UsimagToolConsole::CurvatureAnisotropicFilter(void ) 
{

  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  if ( m_InputImage[m_activeinput]->GetLargestPossibleRegion().GetSize()[2] == 1) {
    std::cout << "Hacer casting 3D -> 2D" << std::endl;
    m_Curvature2D->AddObserver( itk::ProgressEvent(), callback);
    m_Curvature2D->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
    m_Curvature2D->SetNumberOfIterations(static_cast<unsigned int>(AnDifIterations->value()));
    m_Curvature2D->SetConductanceParameter(AnDifConductance->value());
    m_Curvature2D->SetTimeStep(AnDifTimeStep->value());

    m_3Dto2DCaster->SetInput(m_InputImage[m_activeinput]);
    m_3Dto2DCaster->Update();
    m_Curvature2D->SetInput(m_3Dto2DCaster->GetOutput());
    m_Curvature2D->Update();

    m_2Dto3DCaster->SetInput(m_Curvature2D->GetOutput());
    
    if (m_destino->value() == 0) {
      m_OutputImage = m_2Dto3DCaster->GetOutput();
    } else {
      m_InputImage[m_destino->value()-1] = m_2Dto3DCaster->GetOutput();
      this->ShowImageSrc(m_destino->value()-1);
    }

    try {
      m_2Dto3DCaster->Update();
      m_Curvature2D->Modified();
    } catch(itk::ExceptionObject &e) {
      std::cerr << "Caught ITK exception: " << e << std::endl;
    }
  } else {
    m_Curvature->AddObserver( itk::ProgressEvent(), callback);
    m_Curvature->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
    m_Curvature->SetNumberOfIterations(static_cast<unsigned int>(AnDifIterations->value()));
    m_Curvature->SetConductanceParameter(AnDifConductance->value());
    m_Curvature->SetTimeStep(AnDifTimeStep->value());
    

    m_Curvature->SetInput(m_InputImage[m_activeinput]);
    m_Curvature->Update();
    m_Curvature->Modified();

    if (m_destino->value() == 0) {
      m_OutputImage = m_Curvature->GetOutput();
    } else {
      m_InputImage[m_destino->value()-1] = m_Curvature->GetOutput();
      this->ShowImageSrc(m_destino->value()-1);
    }

    try {
      m_Curvature->Update();
    } catch(itk::ExceptionObject &e) {
      std::cerr << "Caught ITK exception: " << e << std::endl;
    }
  }
  progressSlider->value(0);
  m_ProgressCounter->value(0);


}

void UsimagToolConsole::GradientDiffusionFilter(void ) 
{
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_GradientDiffusionFilter->AddObserver( itk::ProgressEvent(), callback);
  m_GradientDiffusionFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_GradientDiffusionFilter->SetNumberOfIterations(static_cast<unsigned int>(GradDifIterations->value()));
  m_GradientDiffusionFilter->SetConductanceParameter(GradDifConductance->value());
  m_GradientDiffusionFilter->SetTimeStep(GradDifTimeStep->value());
  m_GradientDiffusionFilter->SetInput(m_InputImage[m_activeinput]);

  if (m_destino->value() == 0) {
    m_OutputImage = m_GradientDiffusionFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_GradientDiffusionFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  try {
    m_GradientDiffusionFilter->Update();
    m_GradientDiffusionFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);

}

void UsimagToolConsole::GaussianFilter(void ) 
{
  std::cout << "Doing Gaussian Filter" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  m_GaussianFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_GaussianFilter->AddObserver( itk::ProgressEvent(), callback);
  m_GaussianFilter->SetSigma(sigma->value());
  
  // Connect the pipeline
  m_GaussianFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_GaussianFilter->Update(); 
    m_GaussianFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }
  
  if (m_destino->value() == 0) {
    m_OutputImage = m_GaussianFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_GaussianFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::MedianFilter(void ) 
{
  std::cout << "Doing Median Filter" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  InputImageType::SizeType radius;
  radius[0] = (unsigned int)radio0->value();
  radius[1] = (unsigned int)radio1->value();
  radius[2] = (unsigned int)radio2->value();

  m_MedianFilter->AddObserver( itk::ProgressEvent(), callback);
  m_MedianFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_MedianFilter->SetRadius(radius);
  
  // Connect the pipeline
  m_MedianFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_MedianFilter->Update();
    m_MedianFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_MedianFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_MedianFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

//void UsimagToolConsole::WatershedFilter(void ) {
  

//}

void UsimagToolConsole::MultiplyFilter(void ) {
  std::cout << "Haciendo Multiplicacion" << std::endl;
  
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  m_MultiplyFilter->AddObserver( itk::ProgressEvent(), callback);
  if (m_op1->value() == 0) {
    m_MultiplyFilter->SetInput1(m_OutputImage);
  } else { 
    m_MultiplyFilter->SetInput1(m_InputImage[m_op1->value()-1]);
  }
  if (m_op2->value() == 0) {
    m_MultiplyFilter->SetInput2(m_OutputImage);
  } else { 
    m_MultiplyFilter->SetInput2(m_InputImage[m_op2->value()-1]);
  }
  
  try {
    m_MultiplyFilter->Update();
    m_MultiplyFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_MultiplyFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_MultiplyFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::ABSFilter(void ) {
  std::cout << "Haciendo ABS filter" << std::endl;
  
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_ABSFilter->AddObserver( itk::ProgressEvent(), callback);

  try {
    m_ABSFilter->Update();
    m_ABSFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_ABSFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_ABSFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::ABSValDifFilter(void ) {
  std::cout << "Haciendo Diferencia" << std::endl;
  
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  m_ABSValDifFilter->AddObserver( itk::ProgressEvent(), callback);
  if (m_op1->value() == 0) {
    m_ABSValDifFilter->SetInput1(m_OutputImage);
  } else { 
    m_ABSValDifFilter->SetInput1(m_InputImage[m_op1->value()-1]);
  }
  if (m_op2->value() == 0) {
    m_ABSValDifFilter->SetInput2(m_OutputImage);
  } else { 
    m_ABSValDifFilter->SetInput2(m_InputImage[m_op2->value()-1]);
  }
  
  try {
    m_ABSValDifFilter->Update();
    m_ABSValDifFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_ABSValDifFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_ABSValDifFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::InvertFilter(void ) 
{
  std::cout << "Doing Invert Filter" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  
  m_InvertFilter->AddObserver( itk::ProgressEvent(), callback);
  m_InvertFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_InvertFilter->SetMaximum(max_out->value());
  
  // Connect the pipeline
  m_InvertFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_InvertFilter->Update();
    m_InvertFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_InvertFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_InvertFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::ZeroEdgeFilter(void ) 
{
  std::cout << "Doing Zero Crossing Edge detector Filter" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  
  m_ZeroEdgeFilter->AddObserver( itk::ProgressEvent(), callback);
  m_ZeroEdgeFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));

  
  // Connect the pipeline
  m_ZeroEdgeFilter->SetInput(m_InputImage[m_activeinput]);

//Add window with these two parameters
 if((z_noparam->value())==0){
  double Zerror=m_max_error->value();
  if ((Zerror<1)&&(Zerror>0)){
  	m_ZeroEdgeFilter->SetMaximumError (m_max_error->value());
  } 
  else {
	if ((Zerror>=1)){m_ZeroEdgeFilter->SetMaximumError (0.9999);}
        if ((Zerror<=0)){m_ZeroEdgeFilter->SetMaximumError (0.0001);}
  }
  

  m_ZeroEdgeFilter->SetVariance (m_var->value());
 }
  try {
    m_ZeroEdgeFilter->Update();
    m_ZeroEdgeFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_ZeroEdgeFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_ZeroEdgeFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::AddFilter(void ) {
    
  if (m_op1->value() == 0) {
    m_AddFilter->SetInput1(m_OutputImage);
  } else { 
    m_AddFilter->SetInput1(m_InputImage[m_op1->value()-1]);
  }
  if (m_op2->value() == 0) {
    m_AddFilter->SetInput2(m_OutputImage);
  } else { 
    m_AddFilter->SetInput2(m_InputImage[m_op2->value()-1]);
  }

  try {
    m_AddFilter->Update();
    m_AddFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_AddFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_AddFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
    
}

void UsimagToolConsole::RelabelFilter(void ) 
{
  std::cout << "Haciendo Relabel" << std::endl;
  //// Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_RelabelFilter->AddObserver( itk::ProgressEvent(), callback);
  m_RelabelFilter->SetChange(relabel_orig->value(),relabel_result->value()); 
  
  //// Connect the pipeline
  m_RelabelFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_RelabelFilter->Update();
    m_RelabelFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_RelabelFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_RelabelFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
   }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::RescaleFilter(void ) 
{
  std::cout << "Haciendo Rescale" << std::endl;
  //// Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_RescaleFilter->AddObserver( itk::ProgressEvent(), callback);  
  
  //// Connect the pipeline
  m_RescaleFilter->SetInput(m_InputImage[m_activeinput]);
  m_RescaleFilter->SetOutputMaximum(rescale_max->value());
  m_RescaleFilter->SetOutputMinimum(rescale_min->value());

  try {
    m_RescaleFilter->Update();
    m_RescaleFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_RescaleFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_RescaleFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
   }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::ErodeFilter(void ) 
{
  std::cout << "Haciendo Erode" << std::endl;
  //// Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_ErodeFilter->AddObserver( itk::ProgressEvent(), callback);  
  
  //// Connect the pipeline
  m_ErodeFilter->SetInput(m_InputImage[m_activeinput]);
  m_ErodeFilter->SetKernel(m_structuringElement);
  m_ErodeFilter->SetObjectValue(1);

  try {
    m_ErodeFilter->Update();
    m_ErodeFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_ErodeFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_ErodeFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
   }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::DilateFilter(void ) 
{
  std::cout << "Haciendo Dilate" << std::endl;
  //// Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_DilateFilter->AddObserver( itk::ProgressEvent(), callback);  
  
  //// Connect the pipeline
  m_DilateFilter->SetInput(m_InputImage[m_activeinput]);
  m_DilateFilter->SetKernel(m_structuringElement);
  m_DilateFilter->SetObjectValue(1);

  try {
    m_DilateFilter->Update();
    m_DilateFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_DilateFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_DilateFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
   }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::RelabelCompFilter(void ) 
{
  std::cout << "Haciendo Relabel Comp" << std::endl;
  //// Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_RelabelCompFilter->AddObserver( itk::ProgressEvent(), callback);  
  
  //// Connect the pipeline
  m_RelabelCompFilter->SetInput(m_InputImage[m_activeinput]);
 
  try {
    m_RelabelCompFilter->Update();
    m_RelabelCompFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_RelabelCompFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_RelabelCompFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
   }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::ThresholdFilter(void ) 
{
  std::cout << "Haciendo Umbralizado" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  m_ThresholdFilter->AddObserver( itk::ProgressEvent(), callback);
  m_ThresholdFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_ThresholdFilter->SetUpperThreshold((unsigned int)threshold_above->value()); 
  m_ThresholdFilter->SetLowerThreshold((unsigned int)threshold_below->value()); 
  m_ThresholdFilter->SetInsideValue(1);
  m_ThresholdFilter->SetOutsideValue(0);
  
  std::cout << "threshold_above " << (unsigned int)threshold_above->value() << " threshold_below " << (unsigned int)threshold_below->value(); 

  // Connect the pipeline
  m_ThresholdFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_ThresholdFilter->Update();
    m_ThresholdFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_ThresholdFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_ThresholdFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::BilateralFilter(void ) 
{
  std::cout << "Haciendo Filtrado Bilateral" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  double DomainSigma[3];
  DomainSigma[0] = BilatDomainSigma0->value();
  DomainSigma[1] = BilatDomainSigma1->value();
  DomainSigma[2] = BilatDomainSigma2->value();
  
  m_BilateralFilter->AddObserver( itk::ProgressEvent(), callback);
  m_BilateralFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_BilateralFilter->SetRangeSigma(BilatRangeSigma->value()); 
  m_BilateralFilter->SetDomainSigma(DomainSigma); 

  // Connect the pipeline
  m_BilateralFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_BilateralFilter->Update();
    m_BilateralFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_BilateralFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_BilateralFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::MeanFilter(void ) 
{
  std::cout << "Haciendo Filtrado Media" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  InputImageType::SizeType radius;
//  radius[0] = 1;
//  radius[1] = 1;
//  radius[2] = 1;

  radius[0] = (unsigned int)mean_radio0->value();
  radius[1] = (unsigned int)mean_radio1->value();
  radius[2] = (unsigned int)mean_radio2->value();

  m_MeanFilter->AddObserver( itk::ProgressEvent(), callback);
  m_MeanFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_MeanFilter->SetRadius(radius); 

  // Connect the pipeline
  m_MeanFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_MeanFilter->Update();
    m_MeanFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_MeanFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_MeanFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::WienerFilter(void ) 
{
  std::cout << "Haciendo Filtrado Wiener" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_WienerFilter->AddObserver( itk::ProgressEvent(), callback);
  m_WienerFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_WienerFilter->Setlambda(lambda->value());
  m_WienerFilter->Setiterations(static_cast<unsigned int>(iterations->value()));
  m_WienerFilter->SetNoise(bias->value());
  m_WienerFilter->SetBias(noise->value());
  m_WienerFilter->Setgamma(gamma->value());  
  m_WienerFilter->Setopcion(m_filtroWiener-1);
  // Connect the pipeline
  m_WienerFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_WienerFilter->Update();
    m_WienerFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_WienerFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_WienerFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::ASRFilter(void ) 
{
  std::cout << "Haciendo Filtrado ASR" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  if ( m_InputImage[m_activeinput]->GetLargestPossibleRegion().GetSize()[2] == 1) {
    std::cout << "Pasando a 2D " << std::endl;
    m_ASRFilter2D->AddObserver( itk::ProgressEvent(), callback);
    m_ASRFilter2D->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
    m_ASRFilter2D->SetIter(static_cast<unsigned int>(asr_iterations->value()));
    m_ASRFilter2D->SetTimeStep( asr_tau->value() );       
    m_ASRFilter2D->SetBeta( asr_beta->value());
    m_ASRFilter2D->SetSigma( asr_sigma->value() );
    m_ASRFilter2D->SetDifference( asr_s->value() );
  
    m_3Dto2DCaster->SetInput(m_InputImage[m_activeinput]);
    m_3Dto2DCaster->Update();
    m_ASRFilter2D->SetInput(m_3Dto2DCaster->GetOutput());
 
    m_2Dto3DCaster->SetInput(m_ASRFilter2D->GetOutput());
    try {
      m_2Dto3DCaster->Update();
      m_ASRFilter2D->Modified();
    } catch(itk::ExceptionObject &e) {
      std::cerr << "Caught ITK exception: " << e << std::endl;
    }

    if (m_destino->value() == 0) {
      m_OutputImage = m_2Dto3DCaster->GetOutput();
    } else {
      m_InputImage[m_destino->value()-1] = m_2Dto3DCaster->GetOutput();
      this->ShowImageSrc(m_destino->value()-1);
    }

  } else {
    
    m_ASRFilter->AddObserver( itk::ProgressEvent(), callback);
    m_ASRFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
    m_ASRFilter->SetIter(static_cast<unsigned int>(asr_iterations->value()));
    m_ASRFilter->SetTimeStep( asr_tau->value() );       
    m_ASRFilter->SetBeta( asr_beta->value());
    m_ASRFilter->SetSigma( asr_sigma->value() );
    m_ASRFilter->SetDifference( asr_s->value() );

    m_ASRFilter->SetInput(m_InputImage[m_activeinput]);
    try {
      m_ASRFilter->Update();
      m_ASRFilter->Modified();
    } catch(itk::ExceptionObject &e) {
      std::cerr << "Caught ITK exception: " << e << std::endl;
    }

    if (m_destino->value() == 0) {
      m_OutputImage = m_ASRFilter->GetOutput();
    } else {
      m_InputImage[m_destino->value()-1] = m_ASRFilter->GetOutput();
      this->ShowImageSrc(m_destino->value()-1);
    }
  }
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::SRADFilter(void ) 
{
  std::cout << "Haciendo filtrado SRAD" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_SRADFilter->AddObserver( itk::ProgressEvent(), callback);
  m_SRADFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_SRADFilter->SetIter(static_cast<unsigned int>(srad_iterations->value()));
  m_SRADFilter->SetTimeStep( srad_tau->value() );       
  m_SRADFilter->SetBeta( srad_beta->value());
  m_SRADFilter->SetExpConstant( srad_expconstant->value() );
  m_SRADFilter->SetTensorFilter( m_SRADdiffusion );

  // Connect the pipeline
  m_SRADFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_SRADFilter->Update();
    m_SRADFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_SRADFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_SRADFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::DPADFilter(void ) 
{
  std::cout << "Haciendo filtrado DPAD" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_DPADFilter->AddObserver( itk::ProgressEvent(), callback);
  m_DPADFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_DPADFilter->SetIter(static_cast<unsigned int>(dpad_iterations->value()));
  m_DPADFilter->SetTimeStep( dpad_tau->value() );       
  //m_DPADFilter->SetDiffusionTerm( srad_beta->value());
  //m_DPADFilter->SetTensorFilter( m_SRADdiffusion );

  // Connect the pipeline
  m_DPADFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_DPADFilter->Update();
    m_DPADFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_DPADFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_DPADFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  

  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::RegisterMLDFilter(void ) 
{
  std::cout << "Haciendo Registrado MLD" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_regMLD->AddObserver( itk::ProgressEvent(), callback);
  m_regMLD->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  
  m_regMLD->SetInput1( m_InputImage[0]);
  m_regMLD->SetInput2( m_InputImage[1]);

  m_regMLD->SetNLevels( static_cast<int>(reg_nlevels->value()) );
  m_regMLD->SetSteps( static_cast<int>(reg_steps->value()) );
  m_regMLD->SetUseElasticRegularization( reg_elasticreg->value() );
  m_regMLD->SetUseFluidRegularization( reg_fluidreg->value() );
  m_regMLD->SetSigmaElastic( static_cast<int>(reg_sigmaelastic->value()) );
  m_regMLD->SetSigmaFluid( static_cast<int>(reg_sigmafluid->value())  );
  m_regMLD->SetSigmaStats( reg_sigmastats->value()  );
  m_regMLD->SetSigmaGradient( reg_sigmagradient->value() );
 

  try {
    m_regMLD->Start();
    m_regMLD->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_regMLD->GetOutput();
  } else {
    m_InputImage[2] = m_regMLD->GetOutput();
    this->ShowImageSrc(2);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::GetClickPoints(float x, float y)
{
  
  seedX->value(x);
  seedY->value(y);
  m_LevelSetSegmentation->Modified();
  
}

void UsimagToolConsole::KNNFileInputChange() {
  const char *text = KNN_FilePageBrowser->value();
  if (text != NULL && strlen(text) > 0) {    
    KNN_filename = text;
  }
}

void UsimagToolConsole::MEFileInputChange() {
  const char *text = ME_FilePageBrowser->value();
  if (text != NULL && strlen(text) > 0) {    
    ME_filename = text;
  }
}


void UsimagToolConsole::MarcacionElipseInit() {
  std::cout << "Doing MarcacionElipseInit " << ME_filename << std::endl;
  m_vtkPlantillaAjustada->SetFichModelo(ME_filename);
  m_vtkFuncionVerosimilitud->SetRuta(ME_filename);
  m_vtkMarcacionElipsoide->SetNomFichMar(ME_filename);
  std::cout << "Setting values" << std::endl;
  m_vtkPlantillaAjustada->SetJ( static_cast<int>(ME_J->value()));
  m_vtkFuncionVerosimilitud->SetJ( static_cast<int>(ME_J->value()));
  m_vtkPlantillaAjustada->SetK( static_cast<int>(ME_K->value()));
  m_vtkFuncionVerosimilitud->SetK( static_cast<int>(ME_K->value()));
  m_vtkPlantillaAjustada->Setdrmax( static_cast<float>(ME_drmax->value()));
  m_vtkFuncionVerosimilitud->Setdrmax( static_cast<float>(ME_drmax->value()));
  m_vtkFuncionVerosimilitud->SetNg(static_cast<int>(ME_Ng->value()));
  m_vtkPlantillaAjustada->SetUsimagTool(1);
  m_vtkFuncionVerosimilitud->SetUsimagTool(1);
  m_vtkOptimizaContorno->SetUsimagTool(1);

}

void UsimagToolConsole::CalculoModeloVerosimilitud() {

  double m_spac_in[3],m_spac_out[3];
  int i,j;
   
  std::cout << "Doing CalculoModeloVerosimilitud" << std::endl;
   //Hay que incluir vtkPolyData.h, vtkMatrix4x4.h, vtkPoints.h, vtkImageData.h, vtkFloatArray.h (... y quiza alguno mas que se me haya pasado).
   
  vtkPolyData* poli = vtkPolyData::New();

  if (static_cast<int>(ME_Carga->value()) == 1) {
    poli = m_vtkMarcacionElipsoide->GeneraElipsoide( m_vtkMarcacionElipsoide->CargarMarcacionInicial());
  } else {
    // poli = m_vtkMarcacionElipsoide->GeneraElipsoide( ME_Fiducials );
    //m_vtkMarcacionElipsoide->GuardarMarcacionInicial( ME_Fiducials );
  }
   
   vtkImageImport* vtkImporter = vtkImageImport::New();
  m_VTKexporter->SetInput(m_InputImage[m_activeinput]);
  ConnectPipelines(m_VTKexporter, vtkImporter);
  vtkImporter->GetOutput()->Update();

  vtkImporter->GetOutput()->GetSpacing(m_spac_in);
   
  for (i=0;i<3;i++)
     m_spac_out[(i+m_activeinput)%3]=m_spac_in[i]*1000; //Cambio de m. a mm. y correspondencia dimensional.
  
  
  vtkMatrix4x4 *plan=vtkMatrix4x4::New();
  vtkMatrix4x4 *tran=vtkMatrix4x4::New();
  for (i=0;i<4;i++)  {
    for (j=0;j<4;j++) {
      plan->SetElement(i,j,0);
      tran->SetElement(i,j,0);
    }
  }
  plan->SetElement(1,0,-1);
  plan->SetElement(2,1,1);
  plan->SetElement(0,2,-1);
  plan->SetElement(3,3,1);
  tran->SetElement(0,1,-1/m_spac_out[0]);
   tran->SetElement(1,2,1/m_spac_out[1]);
  tran->SetElement(2,0,1/m_spac_out[2]);
  for (i=0;i<3;i++)
     tran->SetElement(i,3,(int)m_InputImage[m_activeinput]->GetLargestPossibleRegion().GetSize()[i]/2);
  tran->SetElement(3,3,1);
   
  //std::cout << "slide number " << ImageViewer[m_activeinput]->GetNumSlices() << std::endl;

  m_vtkPlantillaAjustada->IntroducePlano(plan);
  m_vtkPlantillaAjustada->SetSlth(m_spac_out[2]);
  m_vtkPlantillaAjustada->Setdrmax(m_vtkPlantillaAjustada->GetSlth()*m_vtkPlantillaAjustada->Getdrmax());
  m_vtkPlantillaAjustada->ObtieneParametros(poli);
  

  m_vtkFuncionVerosimilitud->SetP(m_vtkPlantillaAjustada->GetP());
  m_vtkFuncionVerosimilitud->Setdrmax(m_vtkPlantillaAjustada->Getdrmax());
  m_vtkFuncionVerosimilitud->IntroduceMatriza(tran);
  m_vtkFuncionVerosimilitud->IntroducePlano(plan);
  m_vtkFuncionVerosimilitud->Setlimite(m_vtkPlantillaAjustada->Getlimite());
   
  m_vtkFuncionVerosimilitud->IntroduceImagen( vtkImporter->GetOutput(), 0);
  m_vtkFuncionVerosimilitud->IntroduceParam(m_vtkPlantillaAjustada->ObtieneParam());
  m_vtkFuncionVerosimilitud->IntroduceCentro(m_vtkPlantillaAjustada->ObtieneCentro());
  m_vtkFuncionVerosimilitud->Ejecuta();
	
  m_vtkPlantillaAjustada->EstableceMascaraElipsoide(m_vtkFuncionVerosimilitud->DevuelveMascaraElipsoide());
  
  // Renderizar!!!!

}

void UsimagToolConsole::CalculoOptimizacionSA() {
   
   //Quiza incluir vtkStructuredPoints, vtkPolyData, vtkPolyDataWriter
   
   //Ahi va una posibilidad para guardar en fichero:
   vtkPolyData *Poli=vtkPolyData::New();
   vtkPolyDataWriter *PolWri=vtkPolyDataWriter::New();

    std::cout << "Doing OptimizacionSA" << std::endl;
   
  m_vtkOptimizaContorno->SetSuper(0);
  m_vtkOptimizaContorno->SetPeriodMalla(1,0,0);
  m_vtkOptimizaContorno->SetNumEntidades(1);
  m_vtkOptimizaContorno->SetDimensionalidadEstados(1);
  m_vtkOptimizaContorno->SetDimensionalidadMalla(2);
  m_vtkOptimizaContorno->SetIndependencia(0);
  m_vtkOptimizaContorno->SetOrdenMalla(4);
  m_vtkOptimizaContorno->SetOrdenEstados(5);
  m_vtkOptimizaContorno->SetDimMalla(m_vtkFuncionVerosimilitud->GetJ(),m_vtkFuncionVerosimilitud->GetP(),1);

  m_vtkOptimizaContorno->SetDimZ(1,1,1);
  m_vtkOptimizaContorno->SetK(m_vtkFuncionVerosimilitud->GetK());
  m_vtkOptimizaContorno->SetV(2); 
  m_vtkOptimizaContorno->ConstruyeVecindario();
  m_vtkOptimizaContorno->EstableceLE(m_vtkFuncionVerosimilitud->DevuelveLR(),0,0);
	

  m_vtkOptimizaContorno->Llamada(0);
 std::cout << "Finished OptimizacionSA" << std::endl;   
   //Prosigue el almacenamiento en fichero.
   
   Poli=m_vtkPlantillaAjustada->ConstruyeModelo(m_vtkOptimizaContorno->DevuelveRho());
   Poli->Update();
   
   //Todo esto en un if con variable para guardar en fichero.
   PolWri->SetFileName("prueba_resultados.vtk");
   PolWri->SetFileTypeToASCII();
   PolWri->SetInput(Poli);
   PolWri->Write();
   
    std::cout << "Finished FileWriting" << std::endl;
   
   //Renderizacion:
   
  //set MarcacionElipsoide(Poli,$MarcacionElipsoide(color)) [MarcacionElipsoide(Plantilla,$id) ConstruyeModelo [MarcacionElipsoide(Optimiza,$id) DevuelveRho]]
  //set max [MarcacionElipsoide(Plantilla,$id) GetMax]
  //set min [MarcacionElipsoide(Plantilla,$id) GetMin]
  //MarcacionElipsoide(Generador,$id) SetMax [lindex $max 0] [lindex $max 1] [lindex $max 2] [lindex $max 3]
  //MarcacionElipsoide(Generador,$id) SetMin [lindex $min 0] [lindex $min 1] [lindex $min 2] [lindex $min 3]
  //set tipo 0
  //set MarcacionElipsoide(actor,$MarcacionElipsoide(color)) [MarcacionElipsoide(Generador,$id) DibujaModelo $MarcacionElipsoide(Poli,$MarcacionElipsoide(color)) $tipo $MarcacionElipsoide(color)]	
  //MainAddActor $MarcacionElipsoide(actor,$MarcacionElipsoide(color))

  //if { $tipo==1 } { 
  //set MarcacionElipsoide(actor,[expr 1+$MarcacionElipsoide(color)]) [MarcacionElipsoide(Generador,$id) DevuelveBarra]
  //MainAddActor $MarcacionElipsoide(actor,[expr 1+$MarcacionElipsoide(color)]) }

  //RenderAll
  
}

void UsimagToolConsole::GradientFilter(void)
{

  std::cout << "Haciendo Gradiente " << std::endl;  
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

 m_GradientFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_GradientFilter->SetInput( m_InputImage[m_activeinput] );
  m_GradientFilter->AddObserver( itk::ProgressEvent(), callback);
  m_GradientFilter->Update();

  if (m_destino->value() == 0) {
    m_OutputImage = m_GradientFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_GradientFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }

  // Ponemos la barra de progreso a cero  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}


void UsimagToolConsole::GenericFilter(itkFilter* myfilter) {
  GenericImageToImageFilter* filter = new GenericImageToImageFilter;

  //CommandPointer callback = CommandType::New();  
  //callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  filter->SetFilter(myfilter);
  //filter->AddObserver( itk::ProgressEvent(), callback);
  filter->SetInput(m_InputImage[m_activeinput]);
  filter->Update();
  
  if (m_destino->value() == 0) {
    m_OutputImage = (InputImageType*)filter->GetOutput();
   } else {
    m_InputImage[m_destino->value()-1] = (InputImageType*)filter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  progressSlider->value(0);
  m_ProgressCounter->value(0);

} 

void UsimagToolConsole::KnnFilter(void) {
  int x,y,z;
  std::cout << "Haciendo kNN " << std::endl;  
  //  CommandPointer callback = CommandType::New();  
  //  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);
  
  IndexType v_index;
  v_index.Fill(0);
  VoronoiInputType::RegionType diagramRegion;
  diagramRegion.SetIndex(v_index);
  VoronoiInputType::SizeType size;

  //Calculamos el maximo de la imagen de entrada:
  itk::MinimumMaximumImageCalculator< InputImageType >::Pointer Ch1Calculator=itk::MinimumMaximumImageCalculator<  InputImageType  >::New();
  Ch1Calculator->SetImage(m_InputImage[m_activeinput]);
  Ch1Calculator->ComputeMaximum();
  size[0]=(unsigned int)Ch1Calculator->GetMaximum()+1;
  if (size[0] < 256) size[0] = 256;
  diagramRegion.SetSize(size);
  std::cout << "inputRegion del Test = " << size << std::endl;
  VoronoiInputType::Pointer inputvoronoi = VoronoiInputType::New();

  inputvoronoi->SetRegions(diagramRegion);
  inputvoronoi->Allocate();
  inputvoronoi->FillBuffer(-1);
  //relleno los prototipos:

  
  ////////// Leer prototipos 
  //std::cout << "KNN filename " << KNN_filename << std::endl;
  //typedef itk::SpatialObjectReader<> PointsReaderType;
  //typedef itk::LandmarkSpatialObject< 3 > LandmarkSpatialObjectType;

  //PointsReaderType::Pointer points_reader = PointsReaderType::New();
  //points_reader->SetFileName(KNN_filename);
  //points_reader->Update();
  //PointsReaderType::GroupPointer group = points_reader->GetGroup();
  //PointsReaderType::GroupType::ChildrenListType* children = group->GetChildren();
  //LandmarkSpatialObjectType::Pointer landmarks =
  //  dynamic_cast< LandmarkSpatialObjectType* >((*(children->begin())).GetPointer());
  ////////////////////////////

  classifyKNNCoreType::ListType proto;
  classifyKNNCoreType::NodeType auxNode;
  DiagramImageType::IndexType index;
  char buffer[2048];
  int pclass;
  FILE* fp = fopen((const char*)KNN_filename,"r");
  while (!feof(fp)) {
    fgets(buffer,sizeof(buffer),fp);
    sscanf(buffer,"%f %f %f %d %d", &x,&y,&z, &index[0],&pclass);      
    auxNode.index_intensity=index;
    auxNode.clase = pclass;
    proto.push_back(auxNode);
  }
  fclose(fp);
  
  std::cout << "proto.size() " << proto.size() << std::endl;
  for (int i=0;i<proto.size();i++){
      inputvoronoi->SetPixel(proto[i].index_intensity,i);
  }


  VoronoiFilterType::Pointer voronoi = VoronoiFilterType::New();
  voronoi->SetBackgroundValue(-1);
  voronoi->SetInput(inputvoronoi);
  // prepare this to became a vector
  voronoi->SetK(static_cast<unsigned int>(KNN_K->value()));
  voronoi->Update();

  DiagramImageType::Pointer diagram = DiagramImageType::New();
  diagram = voronoi->GetOutput();
 
  m_KnnFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_KnnFilter->SetDiagram(diagram );
  m_KnnFilter->SetCh1( m_InputImage[m_activeinput] );
  m_KnnFilter->SetPrototipos(proto);
  m_KnnFilter->SetK(static_cast<unsigned int>(KNN_K->value()));

  //m_KnnFilter->AddObserver( itk::ProgressEvent(), callback);
  m_KnnFilter->Update();

  if (m_destino->value() == 0) {
    m_OutputImage = m_KnnFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_KnnFilter->GetOutput();    
    this->ShowImageSrc(m_destino->value()-1);
  }

  // Ponemos la barra de progreso a cero  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}

void UsimagToolConsole::CannyFilter(void){
std::cout << "Haciendo Filtrado Canny" << std::endl;
  // Create the segmentation filter
  CommandPointer callback = CommandType::New();  
  callback->SetCallbackFunction(this,&UsimagToolConsole::OnProgress);

  m_CannyFilter->AddObserver( itk::ProgressEvent(), callback);
  m_CannyFilter->SetNumberOfThreads(static_cast<unsigned int>(Threads->value()));
  m_CannyFilter->SetUpperThreshold(upper_threshold->value());
  m_CannyFilter->SetLowerThreshold(lower_threshold->value());
  
  // Connect the pipeline
  m_CannyFilter->SetInput(m_InputImage[m_activeinput]);
  
  try {
    m_CannyFilter->Update();
    m_CannyFilter->Modified();
  } catch(itk::ExceptionObject &e) {
    std::cerr << "Caught ITK exception: " << e << std::endl;
  }

  if (m_destino->value() == 0) {
    m_OutputImage = m_CannyFilter->GetOutput();
  } else {
    m_InputImage[m_destino->value()-1] = m_CannyFilter->GetOutput();
    this->ShowImageSrc(m_destino->value()-1);
  }
  
  progressSlider->value(0);
  m_ProgressCounter->value(0);
}
