/*=========================================================================

  Program:   UsimagToolConsole
  Language:  C++
  Date:      5-07-2007
  Version:   1.0

  Copyright (c) 2007 Laboratoy of Image Processing, UVA. All rights reserved.
  See http://www.lpi.tel.uva.es/UsimagTool for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE. 

=========================================================================*/

#include "UsimagToolGUI.h"
#include <itkObject.h>
#include <string>
#include <itkImageToImageFilter.h>

namespace itk {
  template<class TObject> class MemberCommand;
  class EventObject;
}

class UsimagToolConsole : public UsimagToolGUI
{
public:
  typedef itk::MemberCommand<UsimagToolConsole> CommandType;
  typedef itk::SmartPointer<CommandType> CommandPointer;
  typedef FastMarchingFilterType::NodeType       NodeType;
  typedef FastMarchingFilterType::NodeContainer  NodeContainer;
  typedef itk::ImageToImageFilter<InputImageType,InputImageType> itkFilter;
  
  UsimagToolConsole();
  virtual ~UsimagToolConsole();
  void Load(void);
  void LoadME(void);
  void Abrir(int filetype);
  void Quit(void);
  void LoadRaw(void);
  void LoadRaw(const char* filename);
  void LoadMeta(void); 
  void LoadVOL(void);
  void LoadJPEG(void);
  void LoadVTK(void);
  void LoadTIFF(void);
  void LoadPNG(void);
  void LoadDicom(void);
  void LoadKNN(void);
  void LoadDicomSeries(void);
  void directorio(const char* file,char* dir);
  void ShowImageSrc(int window);
  void Distribuir(InputImageType::Pointer aux, int window);
  void OnProgress(itk::Object *source, const itk::EventObject &event);
  void OnFileInputChange();
  void OnFileFormatChange();
  void OnTipoProcesadoChange();
  int  DetermineFileFormatFromFileName(const char *testFile);
  void OnPixelTypeChange();
  void OnByteOrderChange();
  void OnTipoFiltradoChange();
  void OnTipoFiltroWienerChange();
  void GetClickPoints(float x, float y);  
  static void ClickSelectCallback(float x, float y, float z, float v, int c, void * args );
  void OnSlice0Change(unsigned int);
  void OnSlice1Change(unsigned int);
  void OnSlice2Change(unsigned int);
  void OnSlice3Change(unsigned int);

  void GradientFilter(void);
  void ABSFilter(void);
  void ABSValDifFilter(void);
  void BilateralFilter(void);
  void ThresholdLevelSetSegmentation(void);
  void CurvatureAnisotropicFilter(void); 
  void GradientDiffusionFilter(void); 
  void GaussianFilter(void);
  void MedianFilter(void);
  void MeanFilter(void);
  void WienerFilter(void);
  void ASRFilter(void);
  void SRADFilter(void);
  void DPADFilter(void);

  void RegisterMLDFilter(void);
  //void WatershedFilter(void);
  void ThresholdFilter(void);
  void RelabelFilter(void);
  void RescaleFilter(void);
  void ErodeFilter(void);
  void DilateFilter(void);
  void RelabelCompFilter(void);
  void AddFilter(void);
  void MultiplyFilter(void);
  void UpdateViewerAfterIteration();
  void ResetAllParameters();
  void InvertFilter(void);
  void ZeroEdgeFilter(void);

  void MEFileInputChange(void);
  void MarcacionElipseInit(void);
  void CalculoModeloVerosimilitud(void);
  void CalculoOptimizacionSA(void);

  void GenericFilter(itkFilter*);
  void KnnFilter();
  void KNNFileInputChange(void);

  void CannyFilter(void);

  //typedef itk::Object CommandType;
  //typedef itk::Object::Pointer CommandPointer;
  FloatImageType2D::IndexType   m_seedPosition;
  NodeType                      m_node;
  NodeContainer::Pointer        m_seeds;
  int                           m_filetype;
  int                           m_tipofiltrado;
  int                           m_filtroWiener;
private:
  static const int m_FORMAT_COUNT = 8; 
  std::string m_FileFormatPattern[m_FORMAT_COUNT];
};

